@@ -8,16 +8,16 @@
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public
 # License as published by the Free Software Foundation; either
-# version 2.1 of the License, or (at your option) any later version.
+# version 2 of the License, or (at your option) any later version.
 #
-# This software is distributed in the hope that it will be useful,
+# This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # General Public License for more details.
 #
 # You should have received a copy of the GNU General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# License along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 #
 
 import os
@@ -37,8 +37,10 @@ COMMENT_CHAR = '#'
 TBLHDR_POS = 1
 
 # 内部モード識別子(処理中の表を識別)
+M_NODE        = 'Node'
 M_PROPERTY    = 'Property'
 M_RSCDEFAULTS = 'RscDefaults'
+M_OPDEFAULTS  = 'OpDefaults'
 M_RESOURCES   = 'Resources'
 M_ATTRIBUTES  = 'Attributes'
 M_PRIMITIVE   = 'Primitive'
@@ -48,8 +50,10 @@ M_COLOCATION  = 'Colocation'
 M_ORDER       = 'Order'
 # {表ヘッダ文字列: 内部モード識別子}
 MODE_TBL = {
+  'node':            M_NODE,
   'property':        M_PROPERTY,
   'rsc_defaults':    M_RSCDEFAULTS,
+  'op_defaults':     M_OPDEFAULTS,
   'resources':       M_RESOURCES,
   'rsc_attributes':  M_ATTRIBUTES,
   'primitive':       M_PRIMITIVE,
@@ -68,8 +72,10 @@ PRIM_MODE = [PRIM_PROP,PRIM_ATTR,PRIM_OPER]
 
 # 必須列名
 RQCLM_TBL = {
+  (M_NODE,None):           ['uname','ptype','name','value'],
   (M_PROPERTY,None):       ['name','value'],
   (M_RSCDEFAULTS,None):    ['name','value'],
+  (M_OPDEFAULTS,None):     ['name','value'],
   (M_RESOURCES,None):      ['resourceitem','id'],
   (M_ATTRIBUTES,None):     ['id','type','name','value'],
   (M_PRIMITIVE,PRIM_PROP): ['id','class','provider','type'],
@@ -81,24 +87,29 @@ RQCLM_TBL = {
   (M_ORDER,None):          ['first-rsc','then-rsc','score']
 }
 # 非必須列名
-CLM_LOCEXPERT = ['role']
+CLM_NODE = ['ntype']
+CLM_LOCEXPERT = ['role','id_spec']
 CLM_COLOCATION = ['rsc-role','with-rsc-role']
 CLM_ORDER = ['first-action','then-action','symmetrical']
 
 # 種別
 RESOURCE_TYPE = ['primitive','group','clone','ms','master']
 ATTRIBUTE_TYPE = ['params','meta']
+NODE_ATTR_TYPE = ['attributes','utilization']
+PRIM_ATTR_TYPE = ['params','meta','utilization','operations']
 # unary_op
 UNARY_OP = ['defined','not_defined']
 
 # INFINITYを示す文字列（出力時に使用）
 SCORE_INFINITY = 'INFINITY'
-# pingd/diskd使用時に生成するcolocationのスコア値
+# ping[d]/diskd使用時に生成するcolocationのスコア値
 SCORE_PD_COLOCATION = SCORE_INFINITY
 # crmファイルに出力するコメント
 COMMENT_TBL = {
+  'node':         '### Cluster Node ###',
   'property':     '### Cluster Option ###',
   'rsc_defaults': '### Resource Defaults ###',
+  'op_defaults':  '### Operations Defaults ###',
   'primitive':    '### Primitive Configuration ###',
   'group':        '### Group Configuration ###',
   'clone':        '### Clone Configuration ###',
@@ -158,7 +169,7 @@ class Crm:
   '''
   def optionParser(self):
     usage = '%prog [options] CSV_FILE'
-    version = '1.1'
+    version = '1.4'
     description = "  character encoding of supported CSV_FILE are 'UTF-8' and 'Shift_JIS'"
     prog = 'pm_crmgen'
     p = OptionParser(usage=usage,version=version,description=description,prog=prog)
@@ -166,7 +177,7 @@ class Crm:
       help='output generated crm-file to the named file (default: stdout)')
     p.add_option('-V',action='count',dest='loglevel',default=Log.ERROR,
       help='turn on debug info. additional instances increase verbosity')
-    s = ' related to the pingd/diskd (in LOCATION table) is NOT generated'
+    s = ' related to the ping[d]/diskd (in LOCATION table) is NOT generated'
     p.add_option('-C',action='store_false',dest='add_colocation',default=True,
       help='colocation constraint' + s)
     p.add_option('-O',action='store_false',dest='add_order',default=True,
@@ -252,13 +263,12 @@ class Crm:
     ITEM_RI = 'resourceitem'
     def is_RI(clm):
       return (self.mode[0] == M_RESOURCES and clm == ITEM_RI)
-    def get_location_clm(clm,pos):
+    def get_location_clm(clm):
       x = clm.split(':')
       if clm.lower().startswith('score:') and len(x) == 2 and x[1]:
-        score = self.score_validate(x[1],pos,clm)
-        if score:
-          return 'score:%s'%score
-        return 'score:%s'%x[1]
+        return 'score:%s'%self.score_validate(x[1])
+      elif clm.lower().startswith('ping:') and len(x) == 3 and x[1] and x[2]:
+        return 'ping:%s:%s'%(x[1],x[2])
       elif clm.lower().startswith('pingd:') and len(x) == 3 and x[1] and x[2]:
         return 'pingd:%s:%s'%(x[1],x[2])
       elif clm.lower().startswith('diskd:') and len(x) == 2 and x[1]:
@@ -292,11 +302,12 @@ class Crm:
         rql.append(data)
         clm = data
       elif self.mode[0] == M_LOCATION and clm not in RQCLM_TBL[self.mode]:
-        x = get_location_clm(data,i)
+        x = get_location_clm(data)
         if x:
           rql.append(x)
           clm = x
-      elif (self.mode[0] == M_LOCEXPERT and clm in CLM_LOCEXPERT or
+      elif (self.mode[0] == M_NODE and clm in CLM_NODE or
+            self.mode[0] == M_LOCEXPERT and clm in CLM_LOCEXPERT or
             self.mode[0] == M_COLOCATION and clm in CLM_COLOCATION or
             self.mode[0] == M_ORDER and clm in CLM_ORDER):
         rql.append(clm)
@@ -372,18 +383,13 @@ class Crm:
       return 'false'
     log.warn_l(u'無効な値 [%s](%s) が設定されています。'%(bool,pos2clm(pos)))
 
-  def score_validate(self,score,pos,clm=None):
+  def score_validate(self,score):
     if not score:
       return
     x = score.lower()
-    if match_score(x):
+    if re.match('^[+-]?(inf|infinity)$',x) is not None:
       return x.replace('infinity',SCORE_INFINITY).replace('inf',SCORE_INFINITY)
-    if clm:
-      log.warn_l(u'列定義に無効なスコア値 [%s](%s) の "%s" が設定されています。'
-                 %(clm,pos2clm(pos),score))
-    else:
-      log.warn_l(
-        u'無効なスコア値 [%s](%s) が設定されています。'%(score,pos2clm(pos)))
+    return score
 
   '''
     crmファイル生成（【CSV】->【XML】->【crmコマンド】）
@@ -403,7 +409,7 @@ class Crm:
       fd = open(self.input,'rU')
     except Exception,msg:
       log.error(u'ファイルのオープンに失敗しました。[%s]'%self.input)
-      log.info(msg)
+      log.error(msg)
       return 1
     try:
       code_infile = detect_char(fd.read())
@@ -414,7 +420,7 @@ class Crm:
       csvReader = csv.reader(fd)
     except Exception,msg:
       log.error(u'ファイルの読み込みに失敗しました。[%s]'%self.input)
-      log.info(msg)
+      log.error(msg)
       return 1
     while True:
       try:
@@ -424,7 +430,7 @@ class Crm:
         break  # 終端
       except Exception,msg:
         log.error(u'ファイルの読み込みに失敗しました。[%s]'%self.input)
-        log.info(msg)
+        log.error(msg)
         return 1
       if not unicode_listitem(csvl,code_infile):
         fd.close()
@@ -473,7 +479,7 @@ class Crm:
       fd.close()
     except Exception,msg:
       log.error(u'ファイルのクローズに失敗しました。[%s]'%self.input)
-      log.info(msg)
+      log.error(msg)
       return 1
     if self.root.hasChildNodes():
       self.xml_debug()
@@ -488,7 +494,11 @@ class Crm:
     return 0
 
   def csv2xml(self,clmd,RIl,csvl):
-    if self.mode[0] == M_PROPERTY:
+    if self.mode[0] == M_NODE:
+      log.debug_l(u'クラスタ・ノード表のデータを処理します。')
+      self.debug_input(clmd,RIl,csvl)
+      self.skip_mode(not self.csv2xml_node(clmd,csvl))
+    elif self.mode[0] == M_PROPERTY:
       log.debug_l(u'クラスタ・プロパティ表のデータを処理します。')
       self.debug_input(clmd,RIl,csvl)
       self.csv2xml_option('property',clmd,csvl)
@@ -496,6 +506,10 @@ class Crm:
       log.debug_l(u'リソース・デフォルト表のデータを処理します。')
       self.debug_input(clmd,RIl,csvl)
       self.csv2xml_option('rsc_defaults',clmd,csvl)
+    elif self.mode[0] == M_OPDEFAULTS:
+      log.debug_l(u'オペレーション・デフォルト表のデータを処理します。')
+      self.debug_input(clmd,RIl,csvl)
+      self.csv2xml_option('op_defaults',clmd,csvl)
     elif self.mode[0] == M_RESOURCES:
       log.debug_l(u'リソース構成表のデータを処理します。')
       self.debug_input(clmd,RIl,csvl)
@@ -527,7 +541,78 @@ class Crm:
     return True
 
   '''
-    クラスタ・プロパティ/リソース・デフォルト表データのXML化
+    クラスタ・ノード表データのXML化
+    [引数]
+      clmd : 列情報（[列名: 列番号]）を保持する辞書
+      csvl : CSVファイル1行分のリスト
+    [戻り値]
+      True  : OK
+      False : NG（フォーマット・エラー）
+  '''
+  def csv2xml_node(self,clmd,csvl):
+    global errflg2; errflg2 = False
+    changed = False
+    uname = csvl[clmd['uname']]
+    if uname:
+      self.attrd = {}
+      self.attrd['uname'] = uname; changed = True
+    else:
+      uname = self.attrd.get('uname')
+      if not uname:
+        log.fmterr_l(u"'uname'列に値が設定されていません。")
+    ntype = ''
+    if 'ntype' in clmd:
+      ntype = csvl[clmd['ntype']]
+      if ntype and changed:
+        self.attrd['ntype'] = ntype
+      elif ntype and not changed and uname:
+        log.warn_l(u"'ntype'列に値が設定されています。")
+        ntype = self.attrd.get('ntype','')
+      elif not ntype and not changed:
+        ntype = self.attrd.get('ntype','')
+    ptype = csvl[clmd['ptype']].lower()
+    if ptype and ptype not in NODE_ATTR_TYPE:
+      log.fmterr_l(u'未定義のパラメータ種別 [ptype: %s] が設定されています。'
+                   %csvl[clmd['ptype']])
+    if not ptype:
+      ptype = self.attrd.get('ptype','')
+    name = csvl[clmd['name']]
+    value = csvl[clmd['value']]
+    if not ptype and (name or value):
+      log.fmterr_l(u"'ptype'列に値が設定されていません。")
+    self.attrd['ptype'] = ptype
+    x = self.xml_get_node(self.root,'nodes')
+    for node in self.xml_get_nodes(x,'node','uname',uname):
+      if node.getAttribute('ntype') == ntype:
+        break
+    else:
+      node = None
+    self.xml_check_nv(node,ptype,name,value)
+    if errflg2:
+      return False
+    #
+    # Example:
+    # <crm>
+    #   <nodes>
+    #     <node uname="pm01" ntype="normal">
+    #       <attributes>
+    #         <nv name="standby" value="off"/>
+    #          :
+    #       <utilization>
+    #         <nv name="capacity" value="1"/>
+    #          :
+    #
+    if not node:
+      node = self.xml_create_child(x,'node')
+      node.setAttribute('uname',uname)
+      if ntype:
+        node.setAttribute('ntype',ntype)
+    if ptype:
+      self.xml_append_nv(self.xml_get_node(node,ptype),name,value)
+    return True
+
+  '''
+    クラスタ・プロパティ/リソース、オペレーション・デフォルト表データのXML化
     [引数]
       tag  : データ（<nv .../>）を追加するNodeのタグ名
       clmd : 列情報（[列名: 列番号]）を保持する辞書
@@ -552,6 +637,9 @@ class Crm:
     #   <rsc_defaults>
     #     <nv name="resource-stickiness" value="INFINITY"/>
     #      :
+    #   <op_defaults>
+    #     <nv name="record-pending" value="true"/>
+    #      :
     #
     return self.xml_append_nv(self.xml_get_node(self.root,tag),name,value)
 
@@ -647,7 +735,7 @@ class Crm:
     [引数]
       clmd : 列情報（[列名: 列番号]）を保持する辞書
       csvl : CSVファイル1行分のリスト
-      node : データ（<params>/<meta>...）を追加するNode
+      node : データ（<params>/<meta>/<utilization>/<operations>...）を追加するNode
              ※Primitiveリソース表の処理時に指定される
     [戻り値]
       True  : OK
@@ -665,9 +753,12 @@ class Crm:
         if not rscid:
           log.fmterr_l(u"'id'列に値が設定されていません。")
       node = self.xml_get_rscnode(rscid)
+      types = ATTRIBUTE_TYPE
+    else:
+      types = PRIM_ATTR_TYPE
     atype = csvl[clmd['type']].lower()
     if atype:
-      if atype in ATTRIBUTE_TYPE:
+      if atype in types:
         self.attrd['type'] = atype
       else:
         log.fmterr_l(u'未定義のパラメータ種別 [type: %s] が設定されています。'
@@ -686,7 +777,7 @@ class Crm:
     # Example:
     # <crm>
     #   <resources>
-    #     <clone id="clnPingd" ...>
+    #     <clone id="clnPing" ...>
     #       <meta>
     #         <nv name="clone-max" value="2"/>
     #          :
@@ -694,6 +785,12 @@ class Crm:
     #       <params>
     #         <nv name="device" value="/dev/xvdb1"/>
     #          :
+    #       <utilization>
+    #         <nv name="capacity" value="1"/>
+    #          :
+    #       <operations>
+    #         <nv name="$id" value="opEx"/>
+    #          :
     #
     return self.xml_append_nv(self.xml_get_node(node,atype),name,value)
 
@@ -739,13 +836,14 @@ class Crm:
     # <crm>
     #   <resources>
     #     <primitive id="prmEx" class="ocf" provider="heartbeat" type="sfex">
-    #       <params>...</params>
-    #       <meta>  ...</meta>
+    #       <params>     ...</params>
+    #       <meta>       ...</meta>
+    #       <utilization>...</utilization>
+    #       <operations> ...</operations>
     #       <op>
     #         <start>
     #           <nv name="interval" value="0s"/>
     #            :
-    #         </start>
     #         <monitor>
     #          :
     #
@@ -795,7 +893,7 @@ class Crm:
     #     <location rsc="grpPg">
     #       <rule type="uname" score="200" node="pm01"/>
     #       <rule type="uname" score="100" node="pm02"/>
-    #       <rule type="pingd" score="-INFINITY" attr="default_ping_set" value="100"/>
+    #       <rule type="ping"  score="-INFINITY" attr="default_ping_set" value="100"/>
     #       <rule type="diskd" score="-INFINITY" attr="diskcheck_status"/>
     #       <rule type="diskd" score="-INFINITY" attr="diskcheck_status_internal"/>
     #     </location>
@@ -811,12 +909,13 @@ class Crm:
         for node in csvl[x].split():
           attr = {'type':'uname','score':k.split(':')[1],'node':node}
           set_attr((l,attr,x))
-      elif (k.startswith('pingd:') or k.startswith('diskd:')) and csvl[x]:
+      elif (k.startswith('ping:') or k.startswith('pingd:') or
+            k.startswith('diskd:')) and csvl[x]:
         if self.bool_validate(csvl[x],x) != 'true':
           continue
         k = k.split(':')
         attr = {'type':k[0],'score':'-%s'%SCORE_INFINITY,'attr':k[1]}
-        if k[0] == 'pingd':
+        if k[0].startswith('ping'):
           attr['value'] = k[2]
         if (not set_attr((l,attr,x)) or
             not self.add_colocation and not self.add_order):
@@ -868,12 +967,25 @@ class Crm:
       False : NG（フォーマット・エラー）
   '''
   def csv2xml_locexpert(self,clmd,csvl):
-    def set_attr(node,names):
+    def has_idspec(tgtref):
+      if 'id_spec' in clmd:
+        x = csvl[clmd['id_spec']]
+        if not tgtref:
+          return x != ''
+        return x and x.startswith('$id-ref=') or x.startswith('id-ref=')
+      return False
+    def set_attr(node,names,tag=None,setflg=False):
+      if tag: e = None
+      else:   e = node
       for k,x in [(k,x) for (k,x) in clmd.items() if k in names and csvl[x]]:
-        node.setAttribute(k,csvl[x])
-      node.setAttribute(self.ATTR_CREATED,str(self.lineno))
+        if not e:
+          e = self.xml_create_child(node,tag)
+        e.setAttribute(k,csvl[x])
+        setflg = True
+      if setflg:
+        e.setAttribute(self.ATTR_CREATED,str(self.lineno))
     global errflg2; errflg2 = False
-    changed = False
+    changed = False; r = None
     rsc = csvl[clmd['rsc']]
     if rsc:
       self.attrd['rsc'] = rsc; changed = True
@@ -882,34 +994,36 @@ class Crm:
       if not rsc:
         log.fmterr_l(u"'rsc'列に値が設定されていません。")
     self.xml_get_rscnode(rsc)
-    if csvl[clmd['score']]:
-      x = self.score_validate(csvl[clmd['score']],clmd['score'])
+    if (has_idspec(True) and
+        not [k for (k,x) in clmd.items() if k not in ['rsc','id_spec'] and csvl[x]]):
+      r = self.doc.createElement('rule')
+    if not r:
+      x = self.score_validate(csvl[clmd['score']])
       if x:
         csvl[clmd['score']] = self.attrd['score'] = x
+        r = self.doc.createElement('rule')
       else:
-        self.attrd['score'] = csvl[clmd['score']]
-      r = self.doc.createElement('rule')
-    else:
-      if changed or not rsc:
-        log.fmterr_l(u"'score'列に値が設定されていません。")
+        if changed or not rsc:
+          log.fmterr_l(u"'score'列に値が設定されていません。")
+        else:
+          if csvl[clmd['bool_op']]:
+            log.warn_l(u"'bool_op'列に値が設定されています。")
+          if 'role' in clmd and csvl[clmd['role']]:
+            log.warn_l(u"'role'列に値が設定されています。")
+          if has_idspec(False):
+            log.warn_l(u"'id_spec'列に値が設定されています。")
+          csvl[clmd['score']] = self.attrd['score']
+      if not csvl[clmd['attribute']]:
+        log.fmterr_l(u"'attribute'列に値が設定されていません。")
+      if not csvl[clmd['op']]:
+        log.fmterr_l(u"'op'列に値が設定されていません。")
       else:
-        if csvl[clmd['bool_op']]:
-          log.warn_l(u"'bool_op'列に値が設定されています。")
-        if 'role' in clmd and csvl[clmd['role']]:
-          log.warn_l(u"'role'列に値が設定されています。")
-        csvl[clmd['score']] = self.attrd['score']
-      r = None
-    if not csvl[clmd['attribute']]:
-      log.fmterr_l(u"'attribute'列に値が設定されていません。")
-    if not csvl[clmd['op']]:
-      log.fmterr_l(u"'op'列に値が設定されていません。")
-    else:
-      if csvl[clmd['op']].lower() in UNARY_OP:
-        if csvl[clmd['value']]:
-          log.warn_l(u"'value'列に値が設定されています。")
-      else:
-        if not csvl[clmd['value']]:
-          log.fmterr_l(u"'value'列に値が設定されていません。")
+        if csvl[clmd['op']].lower() in UNARY_OP:
+          if csvl[clmd['value']]:
+            log.warn_l(u"'value'列に値が設定されています。")
+        else:
+          if not csvl[clmd['value']]:
+            log.fmterr_l(u"'value'列に値が設定されていません。")
     if errflg2:
       return False
     #
@@ -917,7 +1031,7 @@ class Crm:
     # <crm>
     #   <locexperts>
     #     <locexpert rsc="grpPg">
-    #       <rule score="200">
+    #       <rule score="200" id_spec="$id=&quot;r1&quot;">
     #         <exp attribute="#uname" op="eq" value="pm01"/>
     #       </rule>
     #        :
@@ -931,6 +1045,9 @@ class Crm:
     #         <exp attribute="attr1" op="ne" value="val2"/>
     #       </rule>
     #     </locexpert>
+    #     <locexpert rsc="grpPg2">
+    #       <rule id_spec="$id-ref=&quot;r1&quot;"/>
+    #        :
     #
     x = self.xml_get_node(self.root,'locexperts')
     l = self.xml_get_nodes(x,'locexpert','rsc',rsc)
@@ -945,14 +1062,14 @@ class Crm:
       if x:
         x[0].removeAttribute(self.ATTR_STATE)
       r.setAttribute(self.ATTR_STATE,'working')
-      set_attr(r,['score','bool_op','role'])
+      set_attr(r,['score','bool_op','role','id_spec'])
       l.appendChild(r)
     x = self.xml_get_nodes(l,'rule',self.ATTR_STATE,'working')[0]
     if x.getElementsByTagName('exp') and not x.getAttribute('bool_op'):
       log.fmterr_l(u"'bool_op'列に値が設定されていません。(%s行目)"
                    %x.getAttribute(self.ATTR_CREATED))
       return False
-    set_attr(self.xml_create_child(x,'exp'),['attribute','op','value'])
+    set_attr(x,['attribute','op','value'],'exp')
     return True
 
   '''
@@ -970,7 +1087,7 @@ class Crm:
       log.fmterr_l(u"'%s'列に値が設定されていません。"%x)
     self.xml_get_rscnode(csvl[clmd['rsc']])
     self.xml_get_rscnode(csvl[clmd['with-rsc']])
-    x = self.score_validate(csvl[clmd['score']],clmd['score'])
+    x = self.score_validate(csvl[clmd['score']])
     if x:
       csvl[clmd['score']] = x
     if errflg2:
@@ -981,7 +1098,7 @@ class Crm:
     # Example:
     # <crm>
     #   <colocations>
-    #     <colocation score="INFINITY" rsc="grpPg" with-rsc="clnPingd"/>
+    #     <colocation score="INFINITY" rsc="grpPg" with-rsc="clnPing"/>
     #     <colocation score="INFINITY" rsc="grpPg" with-rsc="clnDiskd1"/>
     #     <colocation score="INFINITY" rsc="grpPg" with-rsc="clnDiskd2"/>
     #   </colocations>
@@ -1008,7 +1125,7 @@ class Crm:
       log.fmterr_l(u"'%s'列に値が設定されていません。"%x)
     self.xml_get_rscnode(csvl[clmd['first-rsc']])
     self.xml_get_rscnode(csvl[clmd['then-rsc']])
-    x = self.score_validate(csvl[clmd['score']],clmd['score'])
+    x = self.score_validate(csvl[clmd['score']])
     if x:
       csvl[clmd['score']] = x
     if clmd.get('symmetrical'):
@@ -1023,7 +1140,7 @@ class Crm:
     # Example:
     # <crm>
     #   <orders>
-    #     <order score="0" first-rsc="clnPingd"  then-rsc="grpPg" symmetrical="false"/>
+    #     <order score="0" first-rsc="clnPing"   then-rsc="grpPg" symmetrical="false"/>
     #     <order score="0" first-rsc="clnDiskd1" then-rsc="grpPg" symmetrical="false"/>
     #     <order score="0" first-rsc="clnDiskd2" then-rsc="grpPg" symmetrical="false"/>
     #   </orders>
@@ -1206,9 +1323,9 @@ class Crm:
 
   '''
     指定されたパラメータ(*)を持つPrimitiveリソースの最上位親リソースのidを取得
-    (*) pingdの「name="name" value="default_ping_set"」など
+    (*) ping[d]の「name="name" value="default_ping_set"」など
     [引数]
-      type  : <primitive type="aaa" ...>のtypeの値（{pingd|diskd}）
+      type  : <primitive type="aaa" ...>のtypeの値（{ping[d]|diskd}）
       tag   : <primitive ...><tag><nv .../>のtag（attribute種別 {params|meta}）
       name  : <primitive ...><tag><nv name="yyy" value="zzz"/>のnameの値
       value : <primitive ...><tag><nv name="yyy" value="zzz"/>のvalueの値
@@ -1268,8 +1385,10 @@ class Crm:
   '''
   def xml2crm(self):
     s = [
+      self.xml2crm_node(),
       self.xml2crm_option('property'),
       self.xml2crm_option('rsc_defaults'),
+      self.xml2crm_option('op_defaults'),
       self.xml2crm_resources(['group','clone','ms']),
       self.xml2crm_primitive(),
       self.xml2crm_location(),
@@ -1280,6 +1399,23 @@ class Crm:
       s.remove(None)
     return '\n'.join(s)
 
+  def xml2crm_node(self):
+    #
+    # node <uname>[:<type>]
+    #   [attributes  <param>=<value> [<param>=<value>...]]
+    #   [utilization <param>=<value> [<param>=<value>...]]
+    #
+    s = []; tag = 'node'
+    for x in self.root.getElementsByTagName(tag):
+      y = []; z = []
+      if x.getAttribute('ntype'):
+        y.append(':%s'%x.getAttribute('ntype'))
+      z.append('node %s%s'%(x.getAttribute('uname'),''.join(y)))
+      self.xml2crm_attr(x,z,NODE_ATTR_TYPE)
+      s.append(' \\\n\t'.join(z))
+    if s:
+      return '%s\n%s\n'%(COMMENT_TBL[tag],'\n'.join(s))
+
   def xml2crm_option(self,tag):
     #
     # property
@@ -1287,6 +1423,9 @@ class Crm:
     # /
     # rsc_defaults
     #   <option>=<value> [<option>=<value>...]
+    # /
+    # op_defaults
+    #   <option>=<value> [<option>=<value>...]
     #
     s = []
     for x in self.root.getElementsByTagName(tag):
@@ -1328,9 +1467,11 @@ class Crm:
       return
     #
     # primitive <rsc> [<class>:[<provider>:]]<type>
-    #   [params attr_list]
-    #   [meta   attr_list]
-    #   [op op_type [<attribute>=<value>...]...]]
+    #   [params      attr_list]
+    #   [meta        attr_list]
+    #   [utilization attr_list]
+    #   [operations  id_spec]
+    #     [op op_type [<attribute>=<value>...]...]]
     #
     s = []; tag = 'primitive'
     for p in self.rr.getElementsByTagName(tag):
@@ -1341,7 +1482,8 @@ class Crm:
         z.append(p.getAttribute('provider'))
       z.append(p.getAttribute('type'))
       y.append('primitive %s %s'%(p.getAttribute('id'),':'.join(z)))
-      self.xml2crm_attr(p,y)
+      self.xml2crm_attr(p,y,PRIM_ATTR_TYPE[:-1])
+      self.xml2crm_attr(p,y,PRIM_ATTR_TYPE[-1:])
       for o in [o for x in p.getElementsByTagName('op') for o in x.childNodes]:
         z = []
         for x in o.childNodes:  # <nv>
@@ -1369,10 +1511,10 @@ class Crm:
         t = x.getAttribute('type')
         if t == 'uname':
           s.append('#uname eq %s'%x.getAttribute('node'))
-        elif t in ['pingd','diskd']:
+        elif t in ['ping','pingd','diskd']:
           a = x.getAttribute('attr')
           s.append('not_defined %s or %s '%(a,a))
-          if t == 'pingd':
+          if t.startswith('ping'):
             s.append('lt %s'%x.getAttribute('value'))
           elif t == 'diskd':
             s.append('eq ERROR')
@@ -1382,10 +1524,13 @@ class Crm:
       r = l.getAttribute('rsc')
       s.append('location rsc_location-%s-%d %s'%(r,seq+i+1,r))
       for x in l.getElementsByTagName('rule'):
-        s.append(' \\\n\trule ')
+        s.append(' \\\n\trule')
+        if x.getAttribute('id_spec'):
+          s.append(' %s'%x.getAttribute('id_spec'))
         if x.getAttribute('role'):
-          s.append('$role="%s" '%x.getAttribute('role'))
-        s.append('%s: '%x.getAttribute('score'))
+          s.append(' $role="%s"'%x.getAttribute('role'))
+        if x.getAttribute('score'):
+          s.append(' %s: '%x.getAttribute('score'))
         z = []
         for y in x.getElementsByTagName('exp'):
           o = y.getAttribute('op')
@@ -1441,12 +1586,17 @@ class Crm:
     if s:
       return '%s\n%s'%(COMMENT_TBL[tag],''.join(s))
 
-  def xml2crm_attr(self,node,s):
+  def xml2crm_attr(self,node,s,nodes=None):
     #
     # params attr_list / meta attr_list
     # attr_list :: <attr>=<val> [<attr>=<val>...]
+    # /
+    # attributes attr_list / utilization attr_list
+    # attr_list :: <param>=<value> [<param>=<value>...]
     #
-    for x in self.xml_get_childs(node,ATTRIBUTE_TYPE):
+    if not nodes:
+      nodes = ATTRIBUTE_TYPE
+    for x in self.xml_get_childs(node,nodes):
       for i,y in enumerate(x.childNodes):  # <nv>
         if i == 0:
           s.append(x.nodeName)
@@ -1466,27 +1616,27 @@ class Crm:
         sys.stdout.write(string)
       except Exception,msg:
         log.error(u'crmコマンドの出力に失敗しました。[標準出力]')
-        log.info(msg)
+        log.error(msg)
         return False
     else:
       try:
         fd = codecs.open(self.output,'w',CODE_OUTFILE)
       except Exception,msg:
         log.error(u'ファイルのオープンに失敗しました。[%s]'%self.output)
-        log.info(msg)
+        log.error(msg)
         return False
       try:
         fd.write(string)
       except Exception,msg:
         log.error(u'crmファイルの出力に失敗しました。[%s]'%self.output)
-        log.info(msg)
+        log.error(msg)
         os.unlink(self.output)
         return False
       try:
         fd.close()
       except Exception,msg:
         log.error(u'ファイルのクローズに失敗しました。[%s]'%self.output)
-        log.info(msg)
+        log.error(msg)
         return False
     return True
 
@@ -1543,6 +1693,11 @@ class Log:
   def warn_l(self,msg):
     self.print2e(self.WARN,msg,self.printitem_file,self.printitem_lineno)
 
+  def notice(self,msg):
+    self.print2e(self.NOTICE,msg)
+  def notice_l(self,msg):
+    self.print2e(self.NOTICE,msg,self.printitem_file,self.printitem_lineno)
+
   def info(self,msg):
     self.print2e(self.INFO,msg)
   def info_f(self,msg):
@@ -1569,11 +1724,11 @@ class Log:
   def fmterr_f(self,info=None):
     self.error_f(self.fmterrmsg)
     if info:
-      self.info_f(info)
+      self.error_f(info)
   def fmterr_l(self,info=None):
     self.error_l(self.fmterrmsg)
     if info:
-      self.info_l(info)
+      self.error_l(info)
 
   def quitmsg(self,ret):
     if ret == 1:
@@ -1594,7 +1749,7 @@ class Log:
       level = k + ' ' * (self.level_maxlen - len(k))
     if file:
       if lineno:
-        sys.stderr.write('%s: %s(%s): %s\n'%(level,file,lineno,msg))
+        sys.stderr.write('%s: %s(L%s): %s\n'%(level,file,lineno,msg))
       else:
         sys.stderr.write('%s: %s: %s\n'%(level,file,msg))
     else:
@@ -1609,18 +1764,18 @@ class Log:
     ・要素の前後の全半角空白/タブ/改行を削除
     ・文字列中の改行を半角空白に置換
   [引数]
-    list     : 変換対象のリスト
+    tl       : 変換対象のリスト
     encoding : `encoding' -> Unicode に変換
   [戻り値]
     True  : OK
     False : NG
 '''
-def unicode_listitem(list,encoding):
-  for i,data in [(i,x) for (i,x) in enumerate(list) if x]:
+def unicode_listitem(tl,encoding):
+  for i,data in [(i,x) for (i,x) in enumerate(tl) if x]:
     while data.count('\n\n'):
       data = data.replace('\n\n','\n')
     try:
-      list[i] = del_blank(unicode(data.replace('\n',' '),encoding))
+      tl[i] = del_blank(unicode(data.replace('\n',' '),encoding))
     except:
       log.error(u'データのUnicodeへの変換に失敗しました。')
       return False
@@ -1698,18 +1853,6 @@ def dict2list(d,value_only=None):
     return l
   return [(k,x) for y in l for (k,x) in d.items() if x == y]
 
-'''
-  スコア値の妥当性チェック
-  [引数]
-    score : スコア値
-  [戻り値]
-    True  : OK
-    False : NG
-'''
-def match_score(score):
-  # from: crmコマンド
-  return re.match('^[+-]?(inf|infinity|INFINITY|[[0-9]+)$',score) is not None
-
 '''
   列番号をExcelでの列名に変換
   [引数]
